@@ -5,18 +5,21 @@ __author__ = ["Markus Löning"]
 __all__ = [
     "ShapeletTransformClassifier",
     "MrSEQLClassifier",
+    "ROCKETClassifier",
     "BOSSEnsemble",
     "IndividualBOSS",
-    "KNeighborsTimeSeriesClassifier",
     "TemporalDictionaryEnsemble",
+    "IndividualTDE",
+    "KNeighborsTimeSeriesClassifier",
     "ProximityStump",
     "ProximityTree",
     "ProximityForest",
-    "TimeSeriesForest",
-    "TimeSeriesForestClassifier",
-    "RandomIntervalSpectralForest",
-    "ColumnEnsembleClassifier",
     "ElasticEnsemble",
+    "TimeSeriesForest",
+    "RandomIntervalSpectralForest",
+    "SupervisedTimeSeriesForest",
+    "TimeSeriesForestClassifier",
+    "ColumnEnsembleClassifier",
     "pd",
     "np",
     "load_gunpoint",
@@ -30,7 +33,9 @@ import pandas as pd
 
 from sktime.classification.compose import ColumnEnsembleClassifier
 from sktime.classification.compose import TimeSeriesForestClassifier
-from sktime.classification.dictionary_based import BOSSEnsemble, IndividualBOSS
+from sktime.classification.dictionary_based import BOSSEnsemble
+from sktime.classification.dictionary_based import IndividualBOSS
+from sktime.classification.dictionary_based import IndividualTDE
 from sktime.classification.dictionary_based import TemporalDictionaryEnsemble
 from sktime.classification.distance_based import ElasticEnsemble
 from sktime.classification.distance_based import KNeighborsTimeSeriesClassifier
@@ -38,8 +43,10 @@ from sktime.classification.distance_based import ProximityForest
 from sktime.classification.distance_based import ProximityStump
 from sktime.classification.distance_based import ProximityTree
 from sktime.classification.interval_based import RandomIntervalSpectralForest
+from sktime.classification.interval_based import SupervisedTimeSeriesForest
 from sktime.classification.interval_based import TimeSeriesForest
 from sktime.classification.shapelet_based import MrSEQLClassifier
+from sktime.classification.shapelet_based import ROCKETClassifier
 from sktime.classification.shapelet_based import ShapeletTransformClassifier
 from sktime.datasets import load_arrow_head
 from sktime.datasets import load_basic_motions
