@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
-""" KNN time series classification built on sklearn KNeighborsClassifier
+""" KNN time series classification built on sklearn KNeighborsClassifier that
+supports a range of distance measure specifically for time series. This distance
+functions are defined in cython in sktime.distances.elastic_cython. Python versions
+are in sktime.distances.elastic, but these are orders of magnitude slower.
 
 """
 
@@ -28,14 +31,17 @@ from sklearn.utils._joblib import __version__ as joblib_version
 from sklearn.utils.extmath import weighted_mode
 from sklearn.utils.multiclass import check_classification_targets
 from sklearn.utils.validation import check_array
-from sktime.distances.elastic_cython import ddtw_distance
-from sktime.distances.elastic_cython import dtw_distance
-from sktime.distances.elastic_cython import erp_distance
-from sktime.distances.elastic_cython import lcss_distance
-from sktime.distances.elastic_cython import msm_distance
-from sktime.distances.elastic_cython import twe_distance
-from sktime.distances.elastic_cython import wddtw_distance
-from sktime.distances.elastic_cython import wdtw_distance
+from sktime.distances.elastic import euclidean_distance
+from sktime.distances.elastic_cython import (
+    ddtw_distance,
+    dtw_distance,
+    erp_distance,
+    lcss_distance,
+    msm_distance,
+    twe_distance,
+    wddtw_distance,
+    wdtw_distance,
+)
 
 from sktime.classification.base import BaseClassifier
 from sktime.distances.mpdist import mpdist
@@ -98,6 +104,13 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
 
     """
 
+    # Capabilities: data types this classifier can handle
+    capabilities = {
+        "multivariate": False,
+        "unequal_length": False,
+        "missing_values": False,
+    }
+
     def __init__(
         self,
         n_neighbors=1,
@@ -107,9 +120,22 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
         metric_params=None,
         **kwargs
     ):
+        if algorithm == "kd_tree":
+            raise ValueError(
+                "KNeighborsTimeSeriesClassifier cannot work with kd_tree since kd_tree "
+                "cannot be used with a callable distance metric and we do not support "
+                "precalculated distances as yet."
+            )
+        if algorithm == "ball_tree":
+            raise ValueError(
+                "KNeighborsTimeSeriesClassifier cannot work with ball_tree since "
+                "ball_tree has a list of hard coded distances it can use, and cannot "
+                "work with 3-D arrays"
+            )
 
         self._cv_for_params = False
-
+        if metric == "euclidean":  # Euclidean will default to the base class distance
+            metric = euclidean_distance
         if metric == "dtw":
             metric = dtw_distance
         elif metric == "dtwcv":  # special case to force loocv grid search
@@ -142,28 +168,15 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
             metric = twe_distance
         elif metric == "mpdist":
             metric = mpdist
-        # When mpdist is used, the subsequence length (parameter m) must be set
-        # Example: knn_mpdist = KNeighborsTimeSeriesClassifier(
-        # metric='mpdist', metric_params={'m':30})
+            # When mpdist is used, the subsequence length (parameter m) must be set
+            # Example: knn_mpdist = KNeighborsTimeSeriesClassifier(
+            # metric='mpdist', metric_params={'m':30})
         else:
             if type(metric) is str:
                 raise ValueError(
-                    "Unrecognised distance measure: " + metric + ". Allowed "
-                    "values are "
-                    "names from "
-                    "[dtw,ddtw,"
-                    "wdtw,"
-                    "wddtw,"
-                    "lcss,erp,"
-                    "msm] or "
-                    "please "
-                    "pass a "
-                    "callable "
-                    "distance "
-                    "measure "
-                    "into the "
-                    "constuctor "
-                    "directly."
+                    "Unrecognised distance measure: " + metric + ". Allowed values "
+                    "are names from [euclidean,dtw,ddtw,wdtw,wddtw,lcss,erp,msm] or "
+                    "please pass a callable distance measure into the constuctor"
                 )
 
         super(KNeighborsTimeSeriesClassifier, self).__init__(
@@ -193,8 +206,6 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
         X, y = check_X_y(X, y, enforce_univariate=False, coerce_to_numpy=True)
         y = np.asarray(y)
         check_classification_targets(y)
-
-        # print(X)
         # if internal cv is desired, the relevant flag forces a grid search
         # to evaluate the possible values,
         # find the best, and then set this classifier's params to match
@@ -213,7 +224,7 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
         if y.ndim == 1 or y.ndim == 2 and y.shape[1] == 1:
             if y.ndim != 1:
                 warnings.warn(
-                    "A column-vector y was passed when a 1d array "
+                    "IN TS-KNN: A column-vector y was passed when a 1d array "
                     "was expected. Please change the shape of y to "
                     "(n_samples, ), for example using ravel().",
                     DataConversionWarning,
@@ -241,7 +252,8 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
         else:
             temp = check_array.__code__
             check_array.__code__ = _check_array_ts.__code__
-
+        #  this not fx = self._fit(X, self_y) in order to maintain backward
+        # compatibility with scikit learn 0.23, where _fit does not take an arg y
         fx = self._fit(X)
 
         if hasattr(check_array, "__wrapped__"):
@@ -252,6 +264,13 @@ class KNeighborsTimeSeriesClassifier(_KNeighborsClassifier, BaseClassifier):
         self._is_fitted = True
         return fx
 
+    def _more_tags(self):
+        """Removes the need to pass y with _fit
+        Overrides the scikit learn (>0.23) base class setting where 'requires_y' is true
+        so we can call fx = self._fit(X) and maintain backward compatibility.
+        """
+        return {"requires_y": False}
+
     def kneighbors(self, X, n_neighbors=None, return_distance=True):
         """Finds the K-neighbors of a point.
         Returns indices of and distances to the neighbors of each point.
