@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
-""" experiments.py: method to run experiments as an alternative to orchestration.
-
+""" experiments.py: code to run experiments as an alternative to orchestration.
 
+This file is configured for runs of the main method with command line arguments, or for
+single debugging runs. Results
+are written in a standard format
 TO DO: Tidy up this file!
 """
 
@@ -9,15 +11,35 @@ import os
 
 import sklearn.preprocessing
 import sklearn.utils
-from sklearn.linear_model import RidgeClassifierCV
 
 from sktime.classification.dictionary_based import (
     BOSSEnsemble,
     ContractableBOSS,
     TemporalDictionaryEnsemble,
+    WEASEL,
+    MUSE,
 )
-from sktime.contrib.interval_based._cif import CanonicalIntervalForest
-from sktime.transformations.panel.rocket import Rocket
+from sktime.classification.distance_based import (
+    ElasticEnsemble,
+    ProximityForest,
+    ProximityTree,
+    ProximityStump,
+    KNeighborsTimeSeriesClassifier,
+    ShapeDTW,
+)
+from sktime.classification.hybrid import HIVECOTEV1
+from sktime.classification.hybrid._catch22_forest_classifier import (
+    Catch22ForestClassifier,
+)
+from sktime.classification.interval_based import (
+    TimeSeriesForest,
+    RandomIntervalSpectralForest,
+)
+from sktime.classification.interval_based._cif import CanonicalIntervalForest
+from sktime.classification.interval_based._drcif import DrCIF
+from sktime.classification.shapelet_based import MrSEQLClassifier, ROCKETClassifier
+from sktime.classification.shapelet_based import ShapeletTransformClassifier
+
 
 os.environ["MKL_NUM_THREADS"] = "1"  # must be done before numpy import!!
 os.environ["NUMEXPR_NUM_THREADS"] = "1"  # must be done before numpy import!!
@@ -27,289 +49,112 @@ import sys
 import time
 import numpy as np
 import pandas as pd
-from sklearn import preprocessing
 
+
+from sklearn import preprocessing
 from sklearn.metrics import accuracy_score
 from sklearn.model_selection import cross_val_predict
-from sklearn.preprocessing import FunctionTransformer
-from sklearn.tree import DecisionTreeClassifier
-from statsmodels.tsa.stattools import acf
-
-import sktime.classification.compose._ensemble as ensemble
-import sktime.classification.frequency_based._rise as fb
-import sktime.classification.interval_based._tsf as ib
-import sktime.classification.distance_based._elastic_ensemble as dist
-import sktime.classification.distance_based._time_series_neighbors as nn
-import sktime.classification.distance_based._proximity_forest as pf
-import sktime.classification.shapelet_based._stc as st
 from sktime.utils.data_io import load_from_tsfile_to_dataframe as load_ts
-from sktime.transformations.panel.compose import make_row_transformer
-from sktime.transformations.panel.segment import RandomIntervalSegmenter
+import sktime.datasets.tsc_dataset_names as dataset_lists
 
-from sktime.transformations.panel.reduce import Tabularizer
-from sklearn.pipeline import Pipeline, make_pipeline
-from sklearn.pipeline import FeatureUnion
+__author__ = ["Tony Bagnall"]
 
-__author__ = "Anthony Bagnall"
-
-""" Prototype mechanism for testing classifiers on the UCR format. This mirrors the mechanism used in Java,
+""" Prototype mechanism for testing classifiers on the UCR format. This mirrors the
+mechanism used in Java,
 https://github.com/TonyBagnall/uea-tsc/tree/master/src/main/java/experiments
-but is not yet as engineered. However, if you generate results using the method recommended here, they can be directly
-and automatically compared to the results generated in java
+but is not yet as engineered. However, if you generate results using the method
+recommended here, they can be directly and automatically compared to the results
+generated in java
 
-Will have both low level version and high level orchestration version soon.
 """
 
-
-univariate_datasets = [
-    "ACSF1",
-    "Adiac",
-    "AllGestureWiimoteX",
-    "AllGestureWiimoteY",
-    "AllGestureWiimoteZ",
-    "ArrowHead",
-    "Coffee",
-    "Beef",
-    "BeetleFly",
-    "BirdChicken",
-    "BME",
-    "Car",
-    "CBF",
-    "Chinatown",
-    "ChlorineConcentration",
-    "CinCECGTorso",
-    "Coffee",
-    "Computers",
-    "CricketX",
-    "CricketY",
-    "CricketZ",
-    "Crop",
-    "DiatomSizeReduction",
-    "DistalPhalanxOutlineCorrect",
-    "DistalPhalanxOutlineAgeGroup",
-    "DistalPhalanxTW",
-    "DodgerLoopDay",
-    "DodgerLoopGame",
-    "DodgerLoopWeekend",
-    "Earthquakes",
-    "ECG200",
-    "ECG5000",
-    "ECGFiveDays",
-    "ElectricDevices",
-    "EOGHorizontalSignal",
-    "EOGVerticalSignal",
-    "EthanolLevel",
-    "FaceAll",
-    "FaceFour",
-    "FacesUCR",
-    "FiftyWords",
-    "Fish",
-    "FordA",
-    "FordB",
-    "FreezerRegularTrain",
-    "FreezerSmallTrain",
-    "Fungi",
-    "GestureMidAirD1",
-    "GestureMidAirD2",
-    "GestureMidAirD3",
-    "GesturePebbleZ1",
-    "GesturePebbleZ2",
-    "Ham",
-    "HandOutlines",
-    "Haptics",
-    "Herring",
-    "InlineSkate",
-    "InsectEPGRegularTrain",
-    "InsectEPGSmallTrain",
-    "InsectWingbeatSound",
-    "ItalyPowerDemand",
-    "LargeKitchenAppliances",
-    "Lightning2",
-    "Lightning7",
-    "Mallat",
-    "Meat",
-    "MedicalImages",
-    "MelbournePedestrian",
-    "MiddlePhalanxOutlineCorrect",
-    "MiddlePhalanxOutlineAgeGroup",
-    "MiddlePhalanxTW",
-    "MixedShapesRegularTrain",
-    "MixedShapesSmallTrain",
-    "MoteStrain",
-    "NonInvasiveFetalECGThorax1",
-    "NonInvasiveFetalECGThorax2",
-    "OliveOil",
-    "OSULeaf",
-    "PhalangesOutlinesCorrect",
-    "Phoneme",
-    "PickupGestureWiimoteZ",
-    "PigAirwayPressure",
-    "PigArtPressure",
-    "PigCVP",
-    "PLAID",
-    "Plane",
-    "PowerCons",
-    "ProximalPhalanxOutlineCorrect",
-    "ProximalPhalanxOutlineAgeGroup",
-    "ProximalPhalanxTW",
-    "RefrigerationDevices",
-    "Rock",
-    "ScreenType",
-    "SemgHandGenderCh2",
-    "SemgHandMovementCh2",
-    "SemgHandSubjectCh2",
-    "ShakeGestureWiimoteZ",
-    "ShapeletSim",
-    "ShapesAll",
-    "SmallKitchenAppliances",
-    "SmoothSubspace",
-    "SonyAIBORobotSurface1",
-    "SonyAIBORobotSurface2",
-    "StarlightCurves",
-    "Strawberry",
-    "SwedishLeaf",
-    "Symbols",
-    "SyntheticControl",
-    "ToeSegmentation1",
-    "ToeSegmentation2",
-    "Trace",
-    "TwoLeadECG",
-    "TwoPatterns",
-    "UMD",
-    "UWaveGestureLibraryAll",
-    "UWaveGestureLibraryX",
-    "UWaveGestureLibraryY",
-    "UWaveGestureLibraryZ",
-    "Wafer",
-    "Wine",
-    "WordSynonyms",
-    "Worms",
-    "WormsTwoClass",
-    "Yoga",
-]
-
-multivariate_datasets = [
-    "ArticularyWordRecognition",
-    "AtrialFibrillation",
-    "BasicMotions",
-    "CharacterTrajectories",
-    "Cricket",
-    "DuckDuckGeese",
-    "EigenWorms",
-    "Epilepsy",
-    "EthanolConcentration",
-    "ERing",
-    "FaceDetection",
-    "FingerMovements",
-    "HandMovementDirection",
-    "Handwriting",
-    "Heartbeat",
-    "InsectWingbeat",
-    "JapaneseVowels",
-    "Libras",
-    "LSST",
-    "MotorImagery",
-    "NATOPS",
-    "PenDigits",
-    "PEMS-SF",
-    "PhonemeSpectra",
-    "RacketSports",
-    "SelfRegulationSCP1",
-    "SelfRegulationSCP2",
-    "SpokenArabicDigits",
-    "StandWalkJump",
-    "UWaveGestureLibrary",
+classifier_list = [
+    # Distance based
+    "ProximityForest",
+    "KNeighborsTimeSeriesClassifier",
+    "ElasticEnsemble",
+    "ShapeDTW",
+    # Dictionary based
+    "BOSS",
+    "ContractableBOSS",
+    "TemporalDictionaryEnsemble",
+    "WEASEL",
+    "MUSE",
+    # Interval based
+    "RandomIntervalSpectralForest",
+    "TimeSeriesForest",
+    "CanonicalIntervalForest",
+    # Shapelet based
+    "ShapeletTransformClassifier",
+    "ROCKET",
+    "MrSEQLClassifier",
 ]
 
 
-def set_classifier(cls, resampleId):
+def set_classifier(cls, resampleId=None):
     """
-    Basic way of determining the classifier to build. To differentiate settings just and another elif. So, for example, if
-    you wanted tuned TSF, you just pass TuneTSF and set up the tuning mechanism in the elif.
-    This may well get superceded, it is just how e have always done it
+    Basic way of creating the classifier to build using the default settings. This
+    set up is to help with batch jobs for multiple problems to facilitate easy
+    reproducability. You can set up bespoke classifier in many other ways.
+
     :param cls: String indicating which classifier you want
+    :param resampleId: classifier random seed
+
     :return: A classifier.
 
     """
-    if cls.lower() == "pf":
-        return pf.ProximityForest(random_state=resampleId)
-    elif cls.lower() == "pt":
-        return pf.ProximityTree(random_state=resampleId)
-    elif cls.lower() == "ps":
-        return pf.ProximityStump(random_state=resampleId)
-    elif cls.lower() == "rise":
-        return fb.RandomIntervalSpectralForest(random_state=resampleId)
-    elif cls.lower() == "tsf":
-        return ib.TimeSeriesForest(random_state=resampleId)
-    elif cls.lower() == "cif":
-        return CanonicalIntervalForest(random_state=resampleId)
-    elif cls.lower() == "boss":
+    name = cls.lower()
+    # Distance based
+    if name == "pf" or name == "proximityforest":
+        return ProximityForest(random_state=resampleId)
+    elif name == "pt" or name == "proximitytree":
+        return ProximityTree(random_state=resampleId)
+    elif name == "ps" or name == "proximityStump":
+        return ProximityStump(random_state=resampleId)
+    elif name == "dtwcv" or name == "kneighborstimeseriesclassifier":
+        return KNeighborsTimeSeriesClassifier(metric="dtw")
+    elif name == "ee" or name == "elasticensemble":
+        return ElasticEnsemble()
+    elif name == "shapedtw":
+        return ShapeDTW()
+    # Dictionary based
+    elif name == "boss" or name == "bossensemble":
         return BOSSEnsemble(random_state=resampleId)
-    elif cls.lower() == "cboss":
+    elif name == "cboss" or name == "contractableboss":
         return ContractableBOSS(random_state=resampleId)
-    elif cls.lower() == "tde":
+    elif name == "tde" or name == "temporaldictionaryensemble":
         return TemporalDictionaryEnsemble(random_state=resampleId)
-    elif cls.lower() == "st":
-        return st.ShapeletTransformClassifier(time_contract_in_mins=1500)
-    elif cls.lower() == "dtwcv":
-        return nn.KNeighborsTimeSeriesClassifier(metric="dtwcv")
-    elif cls.lower() == "ee" or cls.lower() == "elasticensemble":
-        return dist.ElasticEnsemble()
-    elif cls.lower() == "tsfcomposite":
-        # It defaults to TSF
-        return ensemble.TimeSeriesForestClassifier()
-    elif cls.lower() == "risecomposite":
-        steps = [
-            ("segment", RandomIntervalSegmenter(n_intervals=1, min_length=5)),
-            (
-                "transform",
-                FeatureUnion(
-                    [
-                        (
-                            "acf",
-                            make_row_transformer(
-                                FunctionTransformer(func=acf_coefs, validate=False)
-                            ),
-                        ),
-                        (
-                            "ps",
-                            make_row_transformer(
-                                FunctionTransformer(func=powerspectrum, validate=False)
-                            ),
-                        ),
-                    ]
-                ),
-            ),
-            ("tabularise", Tabularizer()),
-            ("clf", DecisionTreeClassifier()),
-        ]
-        base_estimator = Pipeline(steps)
-        return ensemble.TimeSeriesForestClassifier(
-            estimator=base_estimator, n_estimators=100
+    elif name == "weasel":
+        return WEASEL(random_state=resampleId)
+    elif name == "muse":
+        return MUSE(random_state=resampleId)
+    # Interval based
+    elif name == "rise" or name == "randomintervalspectralforest":
+        return RandomIntervalSpectralForest(random_state=resampleId)
+    elif name == "tsf" or name == "timeseriesforest":
+        return TimeSeriesForest(random_state=resampleId)
+    elif name == "cif" or name == "canonicalintervalforest":
+        return CanonicalIntervalForest(random_state=resampleId)
+    elif name == "drcif":
+        return DrCIF(random_state=resampleId)
+    # Shapelet based
+    elif name == "stc" or name == "shapelettransformclassifier":
+        return ShapeletTransformClassifier(
+            random_state=resampleId, time_contract_in_mins=1
         )
-    elif cls.lower() == "rocket":
-        rocket_pipeline = make_pipeline(
-            Rocket(random_state=resampleId),
-            RidgeClassifierCV(alphas=np.logspace(-3, 3, 10), normalize=True),
-        )
-        return rocket_pipeline
+    elif name == "mrseql" or name == "mrseqlclassifier":
+        return MrSEQLClassifier(seql_mode="fs", symrep=["sax", "sfa"])
+    elif name == "rocket":
+        return ROCKETClassifier(random_state=resampleId)
+    # Hybrid
+    elif name == "catch22":
+        return Catch22ForestClassifier(random_state=resampleId)
+    elif name == "hivecotev1":
+        return HIVECOTEV1(random_state=resampleId)
     else:
         raise Exception("UNKNOWN CLASSIFIER")
 
 
-def acf_coefs(x, maxlag=100):
-    x = np.asarray(x).ravel()
-    nlags = np.minimum(len(x) - 1, maxlag)
-    return acf(x, nlags=nlags).ravel()
-
-
-def powerspectrum(x, **kwargs):
-    x = np.asarray(x).ravel()
-    fft = np.fft.fft(x)
-    ps = fft.real * fft.real + fft.imag * fft.imag
-    return ps[: ps.shape[0] // 2].ravel()
-
-
 def stratified_resample(X_train, y_train, X_test, y_test, random_state):
     all_labels = np.concatenate((y_train, y_test), axis=None)
     all_data = pd.concat([X_train, X_test])
@@ -370,19 +215,25 @@ def run_experiment(
     train_file=False,
 ):
     """
-    Method to run a basic experiment and write the results to files called testFold<resampleID>.csv and, if required,
-    trainFold<resampleID>.csv.
+    Method to run a basic experiment and write the results to files called
+    testFold<resampleID>.csv and, if required, trainFold<resampleID>.csv.
     :param problem_path: Location of problem files, full path.
-    :param results_path: Location of where to write results. Any required directories will be created
-    :param cls_name: determines which classifier to use, as defined in set_classifier. This assumes predict_proba is
+    :param results_path: Location of where to write results. Any required directories
+        will be created
+    :param cls_name: determines which classifier to use, as defined in set_classifier.
+        This assumes predict_proba is
     implemented, to avoid predicting twice. May break some classifiers though
-    :param dataset: Name of problem. Files must be  <problem_path>/<dataset>/<dataset>+"_TRAIN"+format, same for "_TEST"
-    :param resampleID: Seed for resampling. If set to 0, the default train/test split from file is used. Also used in output file name.
-    :param overwrite: if set to False, this will only build results if there is not a result file already present. If
+    :param dataset: Name of problem. Files must be  <problem_path>/<dataset>/<dataset>+
+                "_TRAIN"+format, same for "_TEST"
+    :param resampleID: Seed for resampling. If set to 0, the default train/test split
+                from file is used. Also used in output file name.
+    :param overwrite: if set to False, this will only build results if there is not a
+                result file already present. If
     True, it will overwrite anything already there
-    :param format: Valid formats are ".ts", ".arff" and ".long". For more info on format, see
-    https://github.com/alan-turing-institute/sktime/blob/master/examples/Loading%20Data%20Examples.ipynb
-    :param train_file: whether to generate train files or not. If true, it performs a 10xCV on the train and saves
+    :param format: Valid formats are ".ts", ".arff" and ".long".
+    For more info on format, see   examples/Loading%20Data%20Examples.ipynb
+    :param train_file: whether to generate train files or not. If true, it performs a
+                10xCV on the train and saves
     :return:
     """
 
@@ -424,16 +275,18 @@ def run_experiment(
         if train_file == False and build_test == False:
             return
 
-    # TO DO: Automatically differentiate between problem types, currently only works with .ts
+    # TO DO: Automatically differentiate between problem types,
+    # currently only works with .ts
     trainX, trainY = load_ts(problem_path + dataset + "/" + dataset + "_TRAIN" + format)
     testX, testY = load_ts(problem_path + dataset + "/" + dataset + "_TEST" + format)
     if resampleID != 0:
         # allLabels = np.concatenate((trainY, testY), axis = None)
         # allData = pd.concat([trainX, testX])
         # train_size = len(trainY) / (len(trainY) + len(testY))
-        # trainX, testX, trainY, testY = train_test_split(allData, allLabels, train_size=train_size,
-        #                                                                random_state=resampleID, shuffle=True,
-        #                                                                stratify=allLabels)
+        # trainX, testX, trainY, testY = train_test_split(allData, allLabels,
+        # train_size=train_size,
+        # random_state=resampleID, shuffle=True,
+        # stratify=allLabels)
         trainX, trainY, testX, testY = stratified_resample(
             trainX, trainY, testX, testY, resampleID
         )
@@ -502,7 +355,7 @@ def run_experiment(
         start = int(round(time.time() * 1000))
         if build_test and hasattr(
             classifier, "_get_train_probs"
-        ):  # Normally Can only do this if test has been built ... well not necessarily true, but will do for now
+        ):  # Normally Can only do this if test has been built
             train_probs = classifier._get_train_probs(trainX)
         else:
             train_probs = cross_val_predict(
@@ -564,10 +417,10 @@ def write_results_to_uea_format(
     class_labels=None,
 ):
     """
-    This is very alpha and I will probably completely change the structure once train fold is sorted, as that internally
+    This is very alpha and I will probably completely change the structure once train
+    fold is sorted, as that internally
     does all this I think!
     Output mirrors that produced by this Java
-    https://github.com/TonyBagnall/uea-tsc/blob/master/src/main/java/experiments/Experiments.java
     :param output_path:
     :param classifier_name:
     :param dataset_name:
@@ -583,7 +436,8 @@ def write_results_to_uea_format(
     """
     if len(actual_class_vals) != len(predicted_class_vals):
         raise IndexError(
-            "The number of predicted class values is not the same as the number of actual class values"
+            "The number of predicted class values is not the same as the "
+            + "number of actual class values"
         )
 
     try:
@@ -619,8 +473,6 @@ def write_results_to_uea_format(
         "w",
     )
 
-    # print(classifier_name+" on "+dataset_name+" for resample "+str(resample_seed)+"   "+train_or_test+" data has line three "+third_line)
-    # the first line of the output file is in the form of:
     # <classifierName>,<datasetName>,<train/test>,<Class Labels>
     file.write(
         str(dataset_name)
@@ -634,23 +486,25 @@ def write_results_to_uea_format(
     )
     file.write("\n")
 
-    # the second line of the output is free form and classifier-specific; usually this will record info
+    # the second line of the output is free form and classifier-specific;
+    # usually this will record info
     # such as parameter options used, any constituent model names for ensembles, etc.
     file.write(str(second_line) + "\n")
 
-    # the third line of the file is the accuracy (should be between 0 and 1 inclusive). If this is a train
-    # output file then it will be a training estimate of the classifier on the training data only (e.g.
-    # 10-fold cv, leave-one-out cv, etc.). If this is a test output file, it should be the output
-    # of the estimator on the test data (likely trained on the training data for a-priori parameter optimisation)
+    # the third line of the file is the accuracy (should be between 0 and 1 inclusive).
+    # If this is a train output file then it will be a training estimate of the
+    # classifier on the training data only (e.g. #10-fold cv, leave-one-out cv, etc.).
+    # If this is a test output file, it should be the output of the estimator on the
+    # test data (likely trained on the training data for a-priori para optimisation)
     file.write(str(third_line))
     file.write("\n")
 
-    # from line 4 onwards each line should include the actual and predicted class labels (comma-separated). If
-    # present, for each case, the probabilities of predicting every class value for this case should also be
-    # appended to the line (a space is also included between the predicted value and the predict_proba). E.g.:
-    #
+    # from line 4 onwards each line should include the actual and predicted class labels
+    # (comma-separated). If present, for each case, the probabilities of predicting
+    # every class value for this case should also be appended to the line (a space is
+    # also included between the predicted value and the predict_proba). E.g.:
     # if predict_proba data IS provided for case i:
-    #   actual_class_val[i], predicted_class_val[i],,prob_class_0[i],prob_class_1[i],...,prob_class_c[i]
+    #   actual_class_val[i], predicted_class_val[i],,
     #
     # if predict_proba data IS NOT provided for case i:
     #   actual_class_val[i], predicted_class_val[i]
@@ -669,9 +523,9 @@ def test_loading():
 
     # test multivariate
     # Test univariate
-    for i in range(0, len(univariate_datasets)):
+    for i in range(0, len(dataset_lists.univariate)):
         data_dir = "E:/tsc_ts/"
-        dataset = univariate_datasets[i]
+        dataset = dataset_lists.univariate[i]
         trainX, trainY = load_ts(data_dir + dataset + "/" + dataset + "_TRAIN.ts")
         testX, testY = load_ts(data_dir + dataset + "/" + dataset + "_TEST.ts")
         print("Loaded " + dataset + " in position " + str(i))
@@ -683,9 +537,9 @@ def test_loading():
         print(testX.shape)
         print("Test Y shape :")
         print(testY.shape)
-    for i in range(16, len(multivariate_datasets)):
+    for i in range(16, len(dataset_lists.multivariate)):
         data_dir = "E:/mtsc_ts/"
-        dataset = multivariate_datasets[i]
+        dataset = dataset_lists.multivariate[i]
         print("Loading " + dataset + " in position " + str(i) + ".......")
         trainX, trainY = load_ts(data_dir + dataset + "/" + dataset + "_TRAIN.ts")
         testX, testY = load_ts(data_dir + dataset + "/" + dataset + "_TEST.ts")
@@ -704,11 +558,6 @@ if __name__ == "__main__":
     """
     Example simple usage, with arguments input via script or hard coded for testing
     """
-
-    #    test_loading()
-    #    sys.exit()
-    #    print('experimenting...')
-    # Input args -dp=${dataDir} -rp=${resultsDir} -cn=${classifier} -dn=${dataset} -f=\$LSB_JOBINDEX
     if sys.argv.__len__() > 1:  # cluster run, this is fragile
         print(sys.argv)
         data_dir = sys.argv[1]
@@ -730,13 +579,14 @@ if __name__ == "__main__":
         #        results_dir = "/scratch/results"
         #         data_dir = "/bench/datasets/Univariate2018/"
         #         results_dir = "C:/Users/ajb/Dropbox/Turing Project/Results/"
+        print(" Local Run")
         data_dir = "C:/Code/sktime/sktime/datasets/data/"
         results_dir = "C:/Temp/"
         #        results_dir = "Z:/Results/sktime Bakeoff/"
-        dataset = "ItalyPowerDemand"
+        dataset = "UnitTest"
         trainX, trainY = load_ts(data_dir + dataset + "/" + dataset + "_TRAIN.ts")
         testX, testY = load_ts(data_dir + dataset + "/" + dataset + "_TEST.ts")
-        classifier = "CIF"
+        classifier = "KNeighborsTimeSeriesClassifier"
         resample = 0
         #         for i in range(0, len(univariate_datasets)):
         #             dataset = univariate_datasets[i]
