@@ -16,9 +16,14 @@ from sktime.base import BaseEstimator
 from sktime.classification.base import BaseClassifier
 from sktime.classification.compose import ColumnEnsembleClassifier
 from sktime.classification.compose import TimeSeriesForestClassifier
+from sktime.classification.dictionary_based import ContractableBOSS
 from sktime.classification.dictionary_based import TemporalDictionaryEnsemble
 from sktime.classification.interval_based import RandomIntervalSpectralForest
+from sktime.classification.interval_based._cif import CanonicalIntervalForest
+from sktime.classification.interval_based._drcif import DrCIF
 from sktime.classification.interval_based import TimeSeriesForest
+from sktime.classification.interval_based import SupervisedTimeSeriesForest
+from sktime.classification.shapelet_based import ROCKETClassifier
 from sktime.classification.shapelet_based import ShapeletTransformClassifier
 from sktime.forecasting.arima import AutoARIMA
 from sktime.forecasting.base import BaseForecaster
@@ -33,6 +38,7 @@ from sktime.forecasting.compose import TransformedTargetForecaster
 from sktime.forecasting.exp_smoothing import ExponentialSmoothing
 from sktime.forecasting.hcrystalball import HCrystalBallForecaster
 from sktime.forecasting.model_selection import ForecastingGridSearchCV
+from sktime.forecasting.model_selection import ForecastingRandomizedSearchCV
 from sktime.forecasting.model_selection import SingleWindowSplitter
 from sktime.forecasting.naive import NaiveForecaster
 from sktime.forecasting.online_learning import OnlineEnsembleForecaster
@@ -66,11 +72,12 @@ from sktime.transformations.series.acf import AutoCorrelationTransformer
 from sktime.transformations.series.acf import PartialAutoCorrelationTransformer
 from sktime.transformations.series.adapt import TabularToSeriesAdaptor
 from sktime.transformations.series.detrend import Detrender
+from sktime.transformations.series.impute import Imputer
+
 
 # The following estimators currently do not pass all unit tests
 # What do they fail? ShapeDTW fails on 3d_numpy_input test, not set up for that
 EXCLUDE_ESTIMATORS = [
-    "ROCKETClassifier",
     "ShapeDTW",
     "HIVECOTEV1",
     "ElasticEnsemble",
@@ -138,6 +145,12 @@ ESTIMATOR_TEST_PARAMS = {
         "param_grid": {"window_length": [2, 5]},
         "scoring": sMAPE(),
     },
+    ForecastingRandomizedSearchCV: {
+        "forecaster": NaiveForecaster(strategy="mean"),
+        "cv": SingleWindowSplitter(fh=1),
+        "param_distributions": {"window_length": [2, 5]},
+        "scoring": sMAPE(),
+    },
     TabularToSeriesAdaptor: {"transformer": StandardScaler()},
     ColumnEnsembleClassifier: {
         "estimators": [
@@ -173,6 +186,7 @@ ESTIMATOR_TEST_PARAMS = {
         "min_shapelet_length": 3,
         "max_shapelet_length": 4,
     },
+    ROCKETClassifier: {"num_kernels": 100},
     TSFreshFeatureExtractor: {"disable_progressbar": True, "show_warnings": False},
     TSFreshRelevantFeatureExtractor: {
         "disable_progressbar": True,
@@ -182,14 +196,18 @@ ESTIMATOR_TEST_PARAMS = {
     TSInterpolator: {"length": 10},
     RandomIntervalSpectralForest: {"n_estimators": 3, "acf_lag": 10, "min_interval": 5},
     SFA: {"return_pandas_data_series": True},
+    ContractableBOSS: {"n_parameter_samples": 25, "max_ensemble_size": 5},
     TemporalDictionaryEnsemble: {
-        "n_parameter_samples": 50,
-        "max_ensemble_size": 10,
-        "randomly_selected_params": 40,
+        "n_parameter_samples": 25,
+        "max_ensemble_size": 5,
+        "randomly_selected_params": 20,
     },
     TimeSeriesForest: {"n_estimators": 3},
     TimeSeriesForestClassifier: {"n_estimators": 3},
     TimeSeriesForestRegressor: {"n_estimators": 3},
+    SupervisedTimeSeriesForest: {"n_estimators": 3},
+    CanonicalIntervalForest: {"n_estimators": 3},
+    DrCIF: {"n_estimators": 3},
     HCrystalBallForecaster: {"model": HoltSmoothingWrapper()},
     BATS: {
         "use_box_cox": False,
@@ -217,6 +235,7 @@ ESTIMATOR_TEST_PARAMS = {
     },
     PartialAutoCorrelationTransformer: {"n_lags": 1},
     AutoCorrelationTransformer: {"n_lags": 1},
+    Imputer: {"method": "mean"},
 }
 
 # These methods should not change the state of the estimator, that is, they should
